@@ -40,7 +40,6 @@ from rattail.db import model, api
 from rattail.gpc import GPC
 from rattail.batch import BatchHandler
 from rattail.csvutil import UnicodeDictReader
-from rattail.util import progress_loop
 from rattail.time import make_utc
 from rattail.config import parse_bool
 
@@ -349,35 +348,44 @@ class LabelBatchHandler(BatchHandler):
         Print some labels!
         """
         rows = self.get_effective_rows(batch)
-        return self.print_labels(batch, rows, progress=progress)
+        self.print_labels(batch, rows, progress=progress)
+        return True
 
     def print_labels(self, batch, rows, progress=None):
         """
         Print all labels for the given batch.
         """
+        label_handler = self.app.get_label_handler()
         profiles = {}
 
         def organize(row, i):
             profile = row.label_profile
+            if not profile:
+                return
             if profile.uuid not in profiles:
                 profiles[profile.uuid] = profile
                 profile.labels = []
 
-            # TODO: should include more things here?
-            data = {}
-            if batch.static_prices:
-                data['price'] = row.regular_price
+            data = row.get_data_dict()
 
-            profile.labels.append((row.product, row.label_quantity, data))
+            # TODO: should rename these columns in the schema; for now
+            # just copy values in the dict to make other logic happy
+            data['sale_starts'] = data['sale_start']
+            data['sale_ends'] = data['sale_end']
 
-        progress_loop(organize, rows, progress,
-                      message="Organizing labels by type")
+            # TODO: this flag seems dubious; not sure how best to
+            # handle, but also not sure it's even useful.  in what
+            # case would we want to print "dynamic" data from product
+            # master when we have it all in the batch?
+            if not batch.static_prices:
+                data['product'] = row.product
+
+            profile.labels.append((data, row.label_quantity))
+
+        self.progress_loop(organize, rows, progress,
+                           message="Organizing labels by type")
 
         # okay now print for real
-        # TODO: this is compatible with legacy code but will re-read all needed
-        # product data from master table instead of levaraging batch rows
-        for profile in profiles.values():
-            printer = profile.get_printer(self.config)
+        for profile in six.itervalues(profiles):
+            printer = label_handler.get_printer(profile)
             printer.print_labels(profile.labels, progress=progress)
-
-        return True
