@@ -77,7 +77,7 @@ class ImportSubcommand(Subcommand):
             return load_object(self.handler_spec)
 
         if self.handler_key:
-            handler = self.app.get_designated_import_handler(self.handler_key)
+            handler = self.app.get_import_handler(self.handler_key)
             return type(handler)
 
         raise RuntimeError("Cannot locate a handler to use!  You "
