@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2017 Lance Edgar
+#  Copyright © 2010-2018 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,8 +24,9 @@
 API for Organizational Models
 """
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
+import six
 from sqlalchemy.orm.exc import NoResultFound
 
 from rattail.db import model
@@ -48,7 +49,7 @@ def get_department(session, key):
       otherwise ``None``.
     """
     # Department.uuid match?
-    department = session.query(model.Department).get(unicode(key))
+    department = session.query(model.Department).get(six.text_type(key))
     if department:
         return department
 
