@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2022 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -94,7 +94,9 @@ class ImportHandler(object):
 
     def __init__(self, config=None, **kwargs):
         self.config = config
-        self.enum = config.get_enum() if config else None
+        if self.config:
+            self.app = self.config.get_app()
+            self.enum = self.config.get_enum()
         self.importers = self.get_importers()
         self.extra_importer_kwargs = kwargs.pop('extra_importer_kwargs', {})
         for key, value in kwargs.items():
