@@ -29,7 +29,6 @@ from __future__ import unicode_literals, absolute_import
 import re
 import time
 import imaplib
-from six.moves import queue
 # import sys
 import logging
 # from traceback import format_exception
@@ -74,20 +73,6 @@ class MailMonitorDaemon(Daemon):
             thread.daemon = True
             thread.start()
 
-            # and an action thread for each monitor profile
-            for profile in account.monitored:
-
-                # establish queue for msg transfer between threads
-                profile.queue = queue.Queue()
-
-                # fire up the action thread
-                name = 'actions-{}'.format(profile.key)
-                log.debug("starting action thread: %s", name)
-                thread = Thread(target=perform_actions, name=name,
-                                args=(self.config, watcher, profile))
-                thread.daemon = True
-                thread.start()
-
         # loop indefinitely.  since this is the main thread, the app
         # will terminate when this method ends; all other threads are
         # "subservient" to this one.
@@ -122,12 +107,14 @@ class IMAPAccountWatcher(object):
         """
         This is the main loop for the account watcher.  It acts as the
         callable target for the thread in which it runs.  It basically
-        checks for new messages, queueing any found, then waits for a
-        spell, then does it again, forever.
+        checks for new messages, processing any found, then waits for
+        a spell, then does it again, forever.
         """
         lastruns = {}
         last_error = None
         while True:
+
+            # nb. each profile represents one folder
             for profile in self.account.monitored:
 
                 # we may have stopped monitoring some profiles
@@ -142,16 +129,16 @@ class IMAPAccountWatcher(object):
                     lastrun = get_lastrun(self.config, profile.key)
                     lastruns[profile.key] = lastrun
 
-                # check if we should poll this one again yet
+                # check if we should process this one again yet
                 now = self.app.make_utc(tzinfo=True)
                 if not lastrun or (now - lastrun).seconds >= profile.imap_delay:
 
-                    # yep, so (try to) poll
+                    # yep, so (try to) process
                     try:
-                        self.poll_profile(profile)
+                        self.process_profile(profile)
 
                     except:
-                        log.exception("failed to poll profile: %s", profile.key)
+                        log.exception("failed to process profile: %s", profile.key)
 
                         # presumably the error has to do with our server
                         # connection, so make sure we re-connect next time
@@ -211,140 +198,135 @@ class IMAPAccountWatcher(object):
         self.server_recycled = self.app.make_utc()
 
     def disconnect(self):
+        if not self.server:
+            return
+
         try:
             self.server.close()
         except:
             log.exception("server.close() failed!")
+        else:
+            try:
+                self.server.logout()
+            except:
+                log.exception("server.logout() failed!")
+        finally:
+            self.server = None
 
-        try:
-            self.server.logout()
-        except:
-            log.exception("server.logout() failed!")
-
-    def poll_profile(self, profile):
+    def process_profile(self, profile, unread_only=None):
+        """
+        Poll a particular IMAP folder and process any messages found.
+        """
         self.connect()
 
+        # select the folder
         result = self.server.select(profile.imap_folder)
         log.debug("IMAP server select (%s) result: %s",
                   profile.imap_folder, result)
 
         # maybe look for "all" or maybe just "unread"
-        if profile.imap_unread_only:
-            criterion = '(UNSEEN)'
-        else:
-            criterion = 'ALL'
-
+        if unread_only is None:
+            unread_only = profile.imap_unread_only
+        criterion = '(UNSEEN)' if unread_only else 'ALL'
         # log.debug("invoking IMAP4.search()")
         code, items = self.server.uid('search', None, criterion)
         if code != 'OK':
             raise RuntimeError("IMAP4.search() returned bad code: {}".format(code))
 
         # config may dictacte a "max batch size" in which case we will
-        # only queue so many messages at a time
+        # only process so many messages at a time
         uids = items[0].split()
         if profile.max_batch_size:
             if len(uids) > profile.max_batch_size:
                 uids = uids[:profile.max_batch_size]
 
-        # add message uids to the queue
+        # process messages
         for uid in uids:
-            profile.queue.put(uid)
+            self.perform_actions(profile, uid)
 
+    def perform_actions(self, profile, msguid):
+        """
+        Perform all configured actions for the given message uid.
+        """
+        # ignore if we've stopped monitoring for this profile
+        if not profile.monitor:
+            return
 
-def perform_actions(config, watcher, profile):
-    """
-    Target for action threads.  Provides the main loop which checks
-    the queue for new messages and invokes actions for each, as they
-    appear.
-    """
-    stop = False
-    while not stop:
+        # log.debug("queue contained a msguid: %s", msguid)
+        for action in profile.actions:
+            try:
+                self.invoke_action(action, msguid)
 
-        # suspend execution briefly, to avoid consuming so much CPU...
-        time.sleep(0.01)
+            except:
+                # stop processing messages altogether for this
+                # profile if it is so configured
+                if profile.stop_on_error:
+                    log.warning("an error was encountered, and config "
+                                "dictates that no more actions should be "
+                                "processed for profile: %s", profile.key)
+                    profile.monitor = False
+                    return
 
-        try:
-            msguid = profile.queue.get_nowait()
-        except queue.Empty:
-            pass
-        except StopProcessing:
-            stop = True
-        else:
-            log.debug("queue contained a msguid: %s", msguid)
-            for action in profile.actions:
-                try:
-                    invoke_action(config, watcher, action, msguid)
+                # either way no more actions should be invoked for
+                # this particular message
+                break
 
-                except:
-                    # stop processing messages altogether for this
-                    # profile if it is so configured
-                    if profile.stop_on_error:
-                        log.warning("an error was encountered, and config "
-                                    "dictates that no more actions should be "
-                                    "processed for profile: %s", profile.key)
-                        stop = True
+    def invoke_action(self, action, msguid):
+        """
+        Invoke a single action on a mail message, retrying as necessary.
+        """
+        attempts = 0
+        errtype = None
+        while True:
+            attempts += 1
+            log.debug("invoking action '%s' (attempt #%s of %s) on file: %s",
+                      action.spec, attempts, action.retry_attempts, msguid)
 
-                    # either way no more actions should be invoked for
-                    # this particular message
-                    break
+            try:
+                action.action(self.server, msguid, *action.args, **action.kwargs)
 
+            except:
 
-def invoke_action(config, watcher, action, msguid):
-    """
-    Invoke a single action on a mail message, retrying as necessary.
-    """
-    attempts = 0
-    errtype = None
-    while True:
-        attempts += 1
-        log.debug("invoking action '%s' (attempt #%s of %s) on file: %s",
-                  action.spec, attempts, action.retry_attempts, msguid)
+                # if we've reached our final attempt, stop retrying
+                if attempts >= action.retry_attempts:
+                    # log.debug("attempt #%s failed for action '%s' (giving up) on "
+                    #           "msguid: %s", attempts, action.spec, msguid,
+                    #           exc_info=True)
+                    log.exception("attempt #%s failed for action '%s' (giving up) on "
+                              "msguid: %s", attempts, action.spec, msguid)
+                    # TODO: add email support
+                    # exc_type, exc, traceback = sys.exc_info()
+                    # send_email(self.config, 'mailmon_action_error', {
+                    #     # 'hostname': socket.gethostname(),
+                    #     # 'path': path,
+                    #     'msguid': msguid,
+                    #     'action': action,
+                    #     'attempts': attempts,
+                    #     'error': exc,
+                    #     'traceback': ''.join(format_exception(exc_type, exc, traceback)).strip(),
+                    # })
+                    raise
 
-        try:
-            action.action(watcher.server, msguid, *action.args, **action.kwargs)
+                # if this exception is not the first, and is of a
+                # different type than seen previously, do *not* continue
+                # to retry
+                if errtype is not None and not isinstance(error, errtype):
+                    log.exception("new exception differs from previous one(s), "
+                                  "giving up on action '%s' for msguid: %s",
+                                  action.spec, msguid)
+                    raise
 
-        except:
+                # record the type of exception seen, and pause for next retry
+                log.warning("attempt #%s failed for action '%s' on msguid: %s",
+                            attempts, action.spec, msguid, exc_info=True)
+                errtype = type(error)
+                log.debug("pausing for %s seconds before making attempt #%s of %s",
+                          action.retry_delay, attempts + 1, action.retry_attempts)
+                if action.retry_delay:
+                    time.sleep(action.retry_delay)
 
-            # if we've reached our final attempt, stop retrying
-            if attempts >= action.retry_attempts:
-                # log.debug("attempt #%s failed for action '%s' (giving up) on "
-                #           "msguid: %s", attempts, action.spec, msguid,
-                #           exc_info=True)
-                log.exception("attempt #%s failed for action '%s' (giving up) on "
-                          "msguid: %s", attempts, action.spec, msguid)
-                # TODO: add email support
-                # exc_type, exc, traceback = sys.exc_info()
-                # send_email(config, 'mailmon_action_error', {
-                #     # 'hostname': socket.gethostname(),
-                #     # 'path': path,
-                #     'msguid': msguid,
-                #     'action': action,
-                #     'attempts': attempts,
-                #     'error': exc,
-                #     'traceback': ''.join(format_exception(exc_type, exc, traceback)).strip(),
-                # })
-                raise
-
-            # if this exception is not the first, and is of a
-            # different type than seen previously, do *not* continue
-            # to retry
-            if errtype is not None and not isinstance(error, errtype):
-                log.exception("new exception differs from previous one(s), "
-                              "giving up on action '%s' for msguid: %s",
-                              action.spec, msguid)
-                raise
-
-            # record the type of exception seen, and pause for next retry
-            log.warning("attempt #%s failed for action '%s' on msguid: %s",
-                        attempts, action.spec, msguid, exc_info=True)
-            errtype = type(error)
-            log.debug("pausing for %s seconds before making attempt #%s of %s",
-                      action.retry_delay, attempts + 1, action.retry_attempts)
-            if action.retry_delay:
-                time.sleep(action.retry_delay)
-
-        else:
-            # no error, invocation successful
-            log.debug("attempt #%s succeeded for action '%s' on msguid: %s",
-                      attempts, action.spec, msguid)
-            break
+            else:
+                # no error, invocation successful
+                log.debug("attempt #%s succeeded for action '%s' on msguid: %s",
+                          attempts, action.spec, msguid)
+                break
